<?php
/*
Plugin Name: Random Post Date
Description: A lightweight plugin that changes the date of your posts to any random date in the past or future, with option to exclude some posts.
Version:     1.0.0
Author:      Joseph Adediji
Author URI:  https://profiles.wordpress.org/josylad
License:     GPL3
License URI: http://www.gnu.org/licenses/gpl-3.0.html
Text Domain: random-post-date
*/

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) {
    exit; // Exit if accessed directly
}

// Register admin menu page
function rndpd_add_menu_page() {
    add_menu_page(
        esc_html__('Randomize Post Dates', 'random-post-date'),
        esc_html__('Randomize Post Dates', 'random-post-date'),
        'manage_options',
        'rndpd_randomize_posts',
        'rndpd_settings_page',
        'dashicons-calendar-alt'
    );
}
add_action('admin_menu', 'rndpd_add_menu_page');

// Register settings
function rndpd_register_settings() {
    register_setting('rndpd_settings', 'rndpd_date1', ['sanitize_callback' => 'sanitize_text_field']);
    register_setting('rndpd_settings', 'rndpd_date2', ['sanitize_callback' => 'sanitize_text_field']);
    register_setting('rndpd_settings', 'rndpd_post_type', ['sanitize_callback' => 'sanitize_text_field']);
    register_setting('rndpd_settings', 'rndpd_set_modified_date', ['sanitize_callback' => 'absint']);
    register_setting('rndpd_settings', 'rndpd_exclude_ids', ['sanitize_callback' => 'rndpd_sanitize_exclude_ids']);
}
add_action('admin_init', 'rndpd_register_settings');

// Sanitize exclude IDs
function rndpd_sanitize_exclude_ids($input) {
    $ids = array_filter(array_map('trim', explode(',', $input)), 'is_numeric');
    return implode(',', $ids);
}

// Enqueue scripts and styles
function rndpd_enqueue_scripts($hook) {
    if ('toplevel_page_rndpd_randomize_posts' !== $hook) {
        return;
    }

    wp_enqueue_script(
        'rndpd-script',
        plugins_url('js/rndpd-scripts.js', __FILE__),
        ['jquery-ui-datepicker'],
        '1.0.0',
        true
    );

    wp_enqueue_style(
        'rndpd-style',
        plugins_url('css/rndpd-styles.css', __FILE__),
        ['wp-jquery-ui-dialog'],
        '1.0.0'
    );
}
add_action('admin_enqueue_scripts', 'rndpd_enqueue_scripts');

// Admin settings page
function rndpd_settings_page() {
    ?>
    <div class="wrap">
        <h1><?php esc_html_e('Randomize Post Dates', 'random-post-date'); ?></h1>
        <form method="post" action="options.php">
            <?php
            settings_fields('rndpd_settings');
            do_settings_sections('rndpd_options');
            ?>

            <table class="form-table">
                <tr>
                    <th scope="row">
                        <label for="rndpd_date1">Start Date</label>
                    </th>
                    <td>
                        <input type="date" id="rndpd_date1" name="rndpd_date1" value="<?php echo esc_attr(get_option('rndpd_date1')); ?>">
                    </td>
                </tr>
                <tr>
                    <th scope="row">
                        <label for="rndpd_date2">End Date</label>
                    </th>
                    <td>
                        <input type="date" id="rndpd_date2" name="rndpd_date2" value="<?php echo esc_attr(get_option('rndpd_date2')); ?>">
                    </td>
                </tr>
                <tr valign="top">
    <th scope="row">Post Type</th>
    <td>
        <input type="text" name="rndpd_post_type" value="<?php echo esc_attr(get_option('rndpd_post_type', 'post')); ?>" class="regular-text" />
        <p class="description">
            Specify the post type to target (e.g., <code>post</code> for blog posts, <code>page</code> for pages, <code>product</code> for WooCommerce products, or any custom post type name). Default is <code>post</code>.
        </p>
    </td>
</tr>
                <tr>
                    <th scope="row">
                        <label for="rndpd_set_modified_date">Set Modified Date</label>
                    </th>
                    <td>
                        <input type="checkbox" id="rndpd_set_modified_date" name="rndpd_set_modified_date" value="1" <?php checked(get_option('rndpd_set_modified_date'), 1); ?>>
                    </td>
                </tr>
                <tr>
                    <th scope="row">
                        <label for="rndpd_exclude_ids">Exclude Post IDs</label>
                    </th>
                    <td>
                        <input type="text" id="rndpd_exclude_ids" name="rndpd_exclude_ids" value="<?php echo esc_attr(get_option('rndpd_exclude_ids')); ?>">
                        <p class="description">Enter post IDs separated by commas.</p>
                    </td>
                </tr>
            </table>

            <?php submit_button('Save Changes'); ?>
        </form>

        <a href="<?php echo esc_url(add_query_arg(['randomize' => '1', 'rndpd_nonce' => wp_create_nonce('rndpd_randomize')], admin_url('admin.php?page=rndpd_randomize_posts'))); ?>" class="button button-primary">
            <?php esc_html_e('Randomize Now', 'random-post-date'); ?>
        </a>
    </div>
    <?php
}

// Randomize post dates
function rndpd_randomize_dates() {
    if (!isset($_GET['randomize']) || !isset($_GET['rndpd_nonce'])) {
        return;
    }

    $nonce = wp_unslash($_GET['rndpd_nonce']);
    $nonce = sanitize_text_field($nonce);
    
    if (!wp_verify_nonce($nonce, 'rndpd_randomize')) {
        return;
    }


    $start_date = get_option('rndpd_date1');
    $end_date = get_option('rndpd_date2');
    $post_type = get_option('rndpd_post_type', 'post');
    $exclude_ids = array_filter(array_map('trim', explode(',', get_option('rndpd_exclude_ids', ''))), 'is_numeric');

    if (!$start_date || !$end_date) {
        return;
    }

    $args = [
        'post_type'      => $post_type,
        'post_status'    => 'publish',
        'posts_per_page' => -1,
        'post__not_in'   => $exclude_ids,
        'fields'         => 'ids',
    ];

    $posts = get_posts($args);

    foreach ($posts as $post_id) {
        $random_date = gmdate('Y-m-d H:i:s', wp_rand(strtotime($start_date), strtotime($end_date)));
        wp_update_post([
            'ID'            => $post_id,
            'post_date'     => $random_date,
            'post_date_gmt' => get_gmt_from_date($random_date),
        ]);

        if (get_option('rndpd_set_modified_date')) {
            wp_update_post([
                'ID'                    => $post_id,
                'post_modified'         => $random_date,
                'post_modified_gmt'     => get_gmt_from_date($random_date),
            ]);
        }
    }

    wp_redirect(admin_url('admin.php?page=rndpd_randomize_posts&success=1'));
    exit;
}
add_action('admin_init', 'rndpd_randomize_dates');